package OpenThoughtTests;

# Pathing so it works in my test environment
use lib "./lib";
use lib "../lib";
use lib "/home/eric/open/openplugin";
use lib "/home/eric/open/xml2hash/lib";

use OpenThought();

1;
