<head>
  <meta HTTP-EQUIV="Pragma" CONTENT="no-cache">
  <meta HTTP-EQUIV="expired" CONTENT="01-Mar-94 00:00:01 GMT">
  <title>Demo App</title>
</head>
<body marginwidth="0" marginheight="0" leftmargin="0" rightmargin="0" topmargin="0" bgcolor="#339999">


<form name="demoForm">
<table align=center width=615 border=0 cellspacing=0 cellpadding=0>
  <tr valign=middle>
    <td align=left colspan="2">
      This is just a very simple demo to show you some of OpenThought's most
      basic functions.  To use it, just follow these steps:
      <ol>
        <li> Click the "Get OS List" button<br>
        <li> Click an OS -- watch the server give you info!<br>
      </ol>
    </td>
</tr>
</table>
<table align=center width=615 border=1 cellspacing=0 cellpadding=0
       bgcolor="#ffffff">
<tr valign=middle>
<td align=left colspan="2" bgcolor="#CC3333"><img src="" width="1" height="2"></td>
</tr>
<tr>
  <td colspan=2 align=center bgcolor="000000">
  <font size=+1 color=#ffffff><div id="app_title">OpenThought Demo: OS Info</div></font>
  </td>
</tr>
</table>
<center>
<table align=center border=0 cellspacing=0 cellpadding=2 width=615>
<tr>
<td bgcolor=#cc3333" align=center>
<table align=center border=0 cellspacing=0 cellpadding=5 width=100%
       bgcolor=#bbbbbb>
<tr>
  <td align=center colspan=2>
  <input type=button name=search value='Get OS List!'
         onClick="parent.CallUrl('demo.pl','run_mode=mode2')" maxlength=30><br>
  </td>
</tr>

<tr>
  <td align=center>
    <b>OS Names</b><br>
    <font size=2>
    <select name="selectlist" size=6 onChange="parent.CallUrl('demo.pl', 'selectlist', 'run_mode=mode3', 'app_title')">
      <option>-- Download the OS List! --</option>
    </select>
    </font>
  </td>
  <td align=center valign=center>
    OS<br>
    <input type=text name=os size=30 onFocus="this.blur()"><br>
    Creator<br>
    <input type=text name=creator size=30 onFocus="this.blur()"><br>
    Notes<br>
    <textarea name=notes onFocus="this.blur()" rows=8 cols=25 wrap=virtual>
    </textarea><br>
    <table border=0>
      <tr>
        <td>
          <input type=checkbox name=free>Free?<br>
          <input type=checkbox name=cool>Cool?<br>
          <input type=checkbox name=goodlooking>Good Looking?<br>
        </td>
      </tr>
    </table>
  </td>
</tr>
</table>

</td></tr></table>
</form>

</center>

</body>
