<html>
<head>
   <meta http-equiv="Pragma" content="no-cache">
   <meta http-equiv="expired" content="01-Mar-94 00:00:01 GMT">
<tmpl_include name="main-template.js">
</head>

  <tmpl_if name="debug">
    <frameset rows="90%,10%,0/*" border="5" frameborder="1" framespacing="5">
  <tmpl_else>
    <frameset rows="100%,0/*" border="0" frameborder="0">
  </tmpl_if>

    <frame src="<tmpl_var name=application_url>?OpenThought=<OpenThought><settings><event>ui</event></settings></OpenThought>"
           name="contentFrame" scrolling="auto"
           frameborder="0" marginheight="0" marginwidth="0"
           onLoad="parent.document.title = parent.contentFrame.document.title">

    <tmpl_if name="debug">
      <frame src="about:blank" name="debugFrame"
             scrolling="auto" border="5" onLoad="DebugFrameInfo()">
    </tmpl_if>

    <frame src="<tmpl_var name=application_url>?OpenThought=<OpenThought><settings><event>blank</event></settings></OpenThought>" name="commFrame"
           scrolling="no" border="0" noresize>

       <noframes>
          <body>
             This application uses the <a
             href="http://OpenThought.net">OpenThought</a> Application
             Environment, and requires a browser which supports frames.  This
             includes Netscape 4.x+, Internet Explorer 4.x+, Mozilla 0.8+,
             Opera 6.x+, and Konqueror 3.x+.
          </body>
       </noframes>
   </frameset>

</html>
